!------------------------------------------------------------------------------------!
! David Julian Kling | 2016 | Johann Wolfgang Goethe-Universität Frankfurt am Main   !
!------------------------------------------------------------------------------------!






module integrator
use params
implicit none

contains
      subroutine TOV_solver
      use rk4,        only : c4_m, c4_P 
      use ODE_system, only : rho, P
       !------------------------------------------------------------!
       ! character-Parameter für Namensgebungen der Ausgabedateien: !
       !------------------------------------------------------------!
        character(len=*), parameter :: string_dat = 'output_TOV_solver/solution_TOV_eq_'
        character(len_string_num) :: string_num
        character( len(string_num) + len(string_dat) + len('.dat') ) :: string_fin
       !------------------------------------------------------------!
        integer   :: i         ! Laufindex
        real(kra) :: stepsize  ! (Adaptiertive) Schrittweite, irgendwann noch implementieren. 
        real(kra) :: rho_n, r_n, m_n, P_n ! Finale Variablen für Output-Datei
        real(kra) :: m_tween, P_tween     ! Test-Variablen für adaptive Schrittweite
        real(kra) :: rho_center, P_center ! für rho_center hat Stern bestimmten
                                          ! Radius R_end und bestimmte Masse M_end
       ! real :: Delta_P_center            ! Separation der TOV-Lösungen der M-R-Real. in P
        real :: Delta_P_center_exp        ! Separation soll logarithmisch sein, weil sonst
                                          ! Punkte zu weit auseinander.
       !---------------------------------------------------------------------------------!
       
      ! Der Fall points_M_R = 1 muss ausgeschlossen werden, sonst Divergenz => benutze max():
      !  Delta_P_center = ( P_fin - P_init ) / real( max(points_M_R-1,1) )
        Delta_P_center_exp = ( log10(P_fin) - log10(P_init) ) / real( max(points_M_R-1,1) )

        m_n = m_init

        open(unit=unit_M_R_rel, file="mass_radius_relation.dat",action='write')
        do i=1,points_M_R
            call get_new_pictures_filename(i, string_num, string_dat, string_fin)

           ! P_center   = P_init + Delta_P_center*(i-1) ! zentraler Druck des Durchlaufs
            P_center   = ten**( log10(P_init)+Delta_P_center_exp*(i-1) )
            rho_center = rho(P_center)                 ! zentrale Dichte des Durchlaufs

            P_n   = P_center
            rho_n = rho_center
            m_n   = m_init    ! Setze Masse wieder auf null, damit bei neuem Zyklus nicht bei
                              ! Gesamtmasse des vorherigen TOV-Sterns anfängt

            stepsize = stepsize_init
    
            r_n = r_init ! muss der while-Bedingung im folgenden Loop wegen vorher festgelegt werden.
    
            open(unit=unit_single, file=string_fin,action='write')
            write(unit_single,*) r_n, rho_n*rho_geom_to_cgs, m_n*m_geom_to_cgs, P_n*P_geom_to_cgs
            do while( (r_n.lt.r_max) .AND. P_n.ge.zero )
    
                !write(6,*) c4_m(r_n, rho_n, stepsize), c4_P(r_n, rho_n, m_n, P_n, stepsize)
                m_tween = m_n + c4_m(r_n, rho_n, stepsize)           ! m_(n+1)
                P_tween = P_n + c4_P(r_n, rho_n, m_n, P_n, stepsize) ! P_(n+1)
    
                !! hier könnten die adaptive Schrittweite-Statements hin, aber nicht notwendig !!
    
                r_n = r_n + stepsize ! n-Variablen Ausgangspunkt für adaptive Schrittweite
                m_n = m_tween
                P_n = P_tween    ! muss vor rho_n kommen!
                rho_n = rho(P_n) ! muss nach P_n kommen! 
    
               ! Schreibe Lösungen für Gitterpunkt in Datei:
                write(unit_single,*) r_n, rho_n*rho_geom_to_cgs, m_n*m_geom_to_cgs, P_n*P_geom_to_cgs

            end do
            close(unit=unit_single)

           ! Nach dem letzten do-loop sind r_n und m_n der Radius bzw. die Masse des Sterns mit
           ! rho_center, bzw. P_center, dies wird nun in eine separate Datei geschrieben, in der Nur
           ! Die Größen der zentrale-Dichte-zentraler-Druck-Masse-Radius-Beziehung stehen.
            write(unit_M_R_rel,*) r_n, rho_center*rho_geom_to_cgs, m_n*m_geom_to_cgs, P_center*P_geom_to_cgs
           ! stepsize = r_n / 10000.0_kra ! neue Schrittweite ist 10000-stel des vorherigen Radius.

        end do
        close(unit=unit_M_R_rel)
    
      end subroutine TOV_solver

      subroutine get_new_pictures_filename(picture_number, string_num, string_dat, string_fin)
      ! Definiert Namen der Ausgabedateien, diese sollen durchnummeriert sein und der Wert eines
      ! entsprechenden Laufindexes soll im Namen der Datei zu finden sein
      implicit none
        integer          :: picture_number
        character(len=*) :: string_dat
        character(len_string_num) :: string_num
        character( len(string_num) + len(string_dat) + len('.dat') ) :: string_fin

        call int_to_char(picture_number, string_num)
        string_fin = string_dat//trim(string_num)//'.dat'
      end subroutine get_new_pictures_filename

      subroutine int_to_char(picture_number,string) 
      ! Speichert integer-Variablen eine character-Variable (für Namen der Ausgabedateien)
      implicit none
        character(len_string_num) :: string  !output
        integer :: picture_number

        write(string,'(i'//len_string//'.'//len_string//')') picture_number !'(iN.K)' N-stelliges Feld, davon K Stellen angegeben
      end subroutine int_to_char


end module integrator




