#------------------------------------------------------------------------------------#
# David Julian Kling | 2016 | Johann Wolfgang Goethe-Universität Frankfurt am Main   #
#------------------------------------------------------------------------------------#
import matplotlib as mpl

import matplotlib.pyplot as plt
import numpy as np
import os             # Um Ordner zu erstellen, falls diese noch nicht existieren
import matplotlib.ticker as ticker
import matplotlib.colors as colors
from matplotlib.colors import LinearSegmentedColormap
from matplotlib.colors import LogNorm
from matplotlib.colors import SymLogNorm
from glob import glob # Um Dateinamen mehrerer Dateien in einem Array zu speichern etc.


#--------------------------------------------------------------------#
# Erstelle die notwendigen Ordner, falls diese noch nicht existieren #
#--------------------------------------------------------------------#
if not os.path.exists('plots'):
    os.makedirs('plots')

if not os.path.exists('plots/density_plots'):
    os.makedirs('plots/density_plots')
if not os.path.exists('plots/mass_shell_plots'):
    os.makedirs('plots/mass_shell_plots')
if not os.path.exists('plots/pressure_plots'):
    os.makedirs('plots/pressure_plots')
if not os.path.exists('plots/m_shell_P_of_r_plots'):
    os.makedirs('plots/m_shell_P_of_r_plots')
#--------------------------------------------------------------------#


#---------------------#
# Definiere Parameter #
#---------------------#
resolution_dpi = 300 # Definiere Qualität der Plots in dpi
norm_quarter_star = LogNorm()
    # Definiere Schriftgrößen etc.:
dotsize_scatter = 28
fontsize_axes_labels = 16
fontsize_title = 16
M_sun = 1.99892 * 10.0**(33.0) # Sonnenmasse, um Masse in Einheiten der Sonnenmasse zu plotten.
#---------------------#


#------------------------#
# Beschriftungen im Plot #
#------------------------#
title_density_plot    = r'relativistic electron-gas'
#title_density_plot    = r'relativistic electron-gas'
title_pressure_plot   = title_density_plot
title_mass_shell_plot = title_density_plot
title_mass_shell_P_plot = title_density_plot
    # Definiere Achsenbeschriftungen:
x_label_r = r'$r\ \!\mathrm{[km]}$'
y_label_density    = r'$\rho(r)\ \!\mathrm{[g/cm^3]}$'
y_label_pressure   = r'$P(r)\ \!\mathrm{[dyn/cm^2]}$'
y_label_mass_shell = r'$m(r)/M_\odot$'
#------------------------#



####################################
# Masse-Radius-Druck-Relation-Plot #
####################################
# lese Daten der M-R-Beziehung ein:
MR_name = "mass_radius_relation"
MRP_c_name = "MR_central_P_relation"
"""
Der folgende Datensatz muss wie folgt formatiert sein:
    Spalte 1 = R | Spalte 2 = rho_center | Spalte 3 = M | Spalte 4 = P_center
Das hiesige Plotting Skript erwartet cgs-Einheiten!
"""
MR_data = np.loadtxt(MR_name+".dat", unpack=True) # lade Datensatz

# Umrechnung in handlichere Einheiten (m(r): kg -> M_sun; r: cm -> km):
MR_data[0] = MR_data[0] / 10.0**5.0  # r [km]
MR_data[2] = MR_data[2] / M_sun      # m [M_sun]
    
    # plotte M-R-Relation und schreibe in Datei:
fig_MR = plt.figure(MR_name)
#plt.plot(MR_data[0],MR_data[2])
plt.scatter(MR_data[0],MR_data[2])

plt.title("mass-radius-relation")
plt.xlabel(r"$R_\star\ \!\mathrm{[km]}$", fontsize=fontsize_axes_labels)
plt.ylabel(r"$M_\star / M_\odot$", fontsize=fontsize_axes_labels)

i_M_max = np.where(MR_data[2] == MR_data[2].max())
#plt.axhline( MR_data[2][i_M_max], color='k', linestyle='--')
#plt.axvline( MR_data[0][i_M_max], color='k', linestyle='--')
#
#textbox_content = '$\quad\ M_\mathrm{max}\ =\mathrm{%.2f}\ \! M_\odot$\n$R(M_\mathrm{max})=\mathrm{%.2f}\ \!\mathrm{km}$'%(MR_data[2][i_M_max], MR_data[0][i_M_max])
#textbox_properties = dict(boxstyle='round', facecolor='wheat', alpha=1.0) 
#ax = plt.gca()
#ax.text(0.7, 0.8, textbox_content, transform=ax.transAxes, fontsize=14,
#verticalalignment='top', bbox=textbox_properties)
    
fig_MR.savefig('plots/'+MR_name+'.png', dpi=resolution_dpi, bbox_inches='tight')
plt.close(fig_MR)

    # plotte M(P_c)-R(P_c)-Relation gemäß TOV-Gl. und schreibe in Datei:
color_RP = 'b'
color_MP = 'r'

fig_MRP = plt.figure(MRP_c_name)
ax_RP = plt.gca()
ax_RP.plot(MR_data[3],MR_data[0],color=color_RP, label="$R_\star$ TOV")
ax_RP.set_xscale('log')

ax_MP = ax_RP.twinx()   # Füge Achse für zweiten Plot hinzu
ax_MP.plot(MR_data[3],MR_data[2],color_MP, label="$M_\star$ TOV")

    #--------------------------------------------#
    # plotte M(P_c)-R(P_c)-Relation gemäß Newton:# 
    #--------------------------------------------#
## lese Daten ein:
#MR_Newton = np.loadtxt("MR_rel_Newtonian_non_rel_NS_Pc_2E31_till_3E33.dat", unpack=True) 
## Umrechnung in handlichere Einheiten (m(r): kg -> M_sun; r: cm -> km):
#MR_Newton[0] = MR_Newton[0] / 10.0**5.0  # r [km]
#MR_Newton[2] = MR_Newton[2] / M_sun      # m [M_sun]
#
#ax_RP.plot(MR_Newton[3],MR_Newton[0],color=color_RP, label="$R_\star$ Newton", linestyle='--')
#ax_MP.plot(MR_Newton[3],MR_Newton[2],color=color_MP, label="$M_\star$ Newton", linestyle='--')
#
## Schaffe Platz für Legende (falls Vergl. mit Newton obere 4 Zeilen, ansonsten die beiden darunter):
##ax_RP.set_ylim(min(np.min(MR_data[0]),np.min(MR_Newton[0])), max(np.max(MR_data[0])*(1.+1.0/8.0),\
##                                                                np.max(MR_Newton[0])*(1.+1.0/8.0)) )
##ax_MP.set_ylim(min(np.min(MR_data[2]),np.min(MR_Newton[0])), max(np.max(MR_data[2])*(1.+1.0/8.0),\
##                                                                np.max(MR_Newton[2])*(1.+1.0/7.0)) )
    #--------------------------------------------#
if isinstance(MR_data[0], (np.ndarray, list)): # für Skalare kein np.min() etc. definiert!
  ax_RP.set_ylim(np.min(MR_data[0]), np.max(MR_data[0])*(1.+1.0/8.0) )
  ax_MP.set_ylim(np.min(MR_data[2]), np.max(MR_data[2])*(1.+1.0/8.0) )
  plt.xlim(0,np.max(MR_data[3]))
  



# definiere Achsenbeschriftungen und Titel des Plots:
ax_RP.set_xlabel(r'$P_\mathrm{c}\ \!\mathrm{[dyn/cm^2]}$',        fontsize=fontsize_axes_labels)
ax_RP.set_ylabel(r'$R_\star\ \!\mathrm{[km]}$',   color=color_RP, fontsize=fontsize_axes_labels)
ax_MP.set_ylabel(r'$M_\star/M_\odot$',            color=color_MP, fontsize=fontsize_axes_labels)
plt.title("mass and radius vs. central pressure",                 fontsize=fontsize_title)

# Füge Legende hinzu:
ax_MP.legend(loc=1)
ax_RP.legend(loc=2)

fig_MRP.savefig('plots/'+MRP_c_name+'.png', dpi=resolution_dpi, bbox_inches='tight')
plt.close(fig_MRP)




# Definiere Maximalwerte für einheitliche Skalierung der folgenden Plots: Druck und Dichte sind im Zentrum am größten => Suche Maximalwerte aller Spalten der Datei "mass_radius_relation.dat"; da dort auch die TOV-Stern-Massen und -Radien drinnen stehen findet man dort alle Werte die man benötigt.
R_max   = np.max(MR_data[0]) 
rho_max = np.max(MR_data[1]) 
M_max   = np.max(MR_data[2]) 
P_max   = np.max(MR_data[3]) 

# speichere Dateinamen der Sequenzen für verschiedene zentrale Bedingungen in Array:
files_list = sorted( glob( os.path.join('output_TOV_solver','*.dat') ) ) 
#---------------------------------------#
# Definiere Namen der aktuellen Sequenz #
#---------------------------------------#
for i in range(0,len(files_list)):
    actual_data_name   = files_list[i] # Name des Datensatzes
    """
    Der folgende Datensatz muss wie folgt formatiert sein:
        Spalte 1 = r | Spalte 2 = rho | Spalte 3 = m | Spalte 4 = P
    Das hiesige Plotting Skript erwartet cgs-Einheiten!
    """
    TOV_data = np.loadtxt(actual_data_name, unpack=True) # lade Datensatz
    
    # Umrechnung in handlichere Einheiten (m(r): kg -> M_sun; r: cm -> km):
    TOV_data[0] = TOV_data[0] / 10.0**5.0  # r [km]
    TOV_data[2] = TOV_data[2] / M_sun      # m [M_sun]
    
    actual_plot_name_rho        = "TOV_density_"+actual_data_name[-9:-4]
    actual_plot_name_mass_shell = "TOV_mass_shell_"+actual_data_name[-9:-4]
    actual_plot_name_P          = "TOV_pressure_"+actual_data_name[-9:-4]
    actual_plot_name_m_shell_P  = "TOV_m_shell_pressure_of_r"+actual_data_name[-9:-4]

    #-------------------#
    # Definiere Legende #
    #-------------------#
    textbox_content = '$\\rho_\mathrm{c}\quad\ \!\! =\mathrm{%.2E}$\n$P_\mathrm{c}\quad\! =\mathrm{%.2E}$\n$M_\star=%.3f\ M_\odot$'%(TOV_data[1][0], TOV_data[3][0], M_max)
    textbox_properties = dict(boxstyle='round', facecolor='wheat', alpha=1.0) 
#---------------------------------------#
    
    
    
    #################################################
    # Druck/Dichte/Massenschalen-Plotting-Sequenzen #
    #################################################
    
    
    
        #-------------#
        # Dichte-Plot #
        #-------------#
    fig_rho = plt.figure(actual_plot_name_rho)
    
    plt.plot(TOV_data[0],TOV_data[1])
    
    plt.title(title_density_plot)
    plt.xlabel(x_label_r,       fontsize=fontsize_axes_labels)
    plt.ylabel(y_label_density, fontsize=fontsize_axes_labels)
    
        # Lege Achsenbereiche fest:
    plt.xlim(0.0,R_max)
    plt.ylim(0.0,rho_max)
    
        # Zeichne Legende:
    ax = plt.gca()
    ax.text(0.75, 0.95, textbox_content, transform=ax.transAxes, fontsize=14,
    verticalalignment='top', bbox=textbox_properties)
        # zeichne vertikale Linie an Stelle des aktuellen Stern-Radius:
    if isinstance(MR_data[0], (np.ndarray, list)):
      ax.axvline(MR_data[0][i], color='k', linestyle='--')

    #plt.show(fig_rho)
    fig_rho.savefig('plots/density_plots/'+actual_plot_name_rho+'.png', dpi=resolution_dpi, bbox_inches='tight')
    plt.close(fig_rho)
    


        #---------------------------#
        # Massenschalen-Druck-Plots #
        #---------------------------#
#Das im folgenden auskommentierte macht Massenschalen- und Druck Plots mit gemeinsamer x-Achse

        # plotte M(P_c)-R(P_c)-Relation gemäß TOV-Gl. und schreibe in Datei:
    color_m_shell = 'r'
    color_P       = 'b'
    
    fig_m_shell_P = plt.figure(actual_plot_name_m_shell_P)
    ax_P = plt.gca()
    ax_P.plot(TOV_data[0], TOV_data[3], color=color_P)
    
    ax_m_shell = ax_P.twinx()   # Füge Achse für zweiten Plot hinzu
    ax_m_shell.plot(TOV_data[0], TOV_data[2], color=color_m_shell)    
    ax_m_shell.set_ylim(0.0, M_max*(1.+1.0/8.0) )
    ax_P.set_ylim(0.0, P_max*(1.+1.0/8.0) )
    plt.xlim(0.0, R_max )
    
    # definiere Achsenbeschriftungen und Titel des Plots:
    ax_m_shell.set_ylabel(y_label_mass_shell, color=color_m_shell, fontsize=fontsize_axes_labels )
    ax_P.set_ylabel(y_label_pressure,         color=color_P,       fontsize=fontsize_axes_labels )
    ax_P.set_xlabel(x_label_r,                               fontsize=fontsize_axes_labels )

    plt.title(title_mass_shell_P_plot, fontsize=fontsize_title)
    
        # zeichne Legende:
    ax = plt.gca()
    ax.text(0.7, 0.8, textbox_content, transform=ax.transAxes, fontsize=14,
    verticalalignment='top', bbox=textbox_properties)
        # zeichne vertikale Linie an Stelle des aktuellen Stern-Radius:
    if isinstance(MR_data[0], (np.ndarray, list)):
      ax.axvline(MR_data[0][i], color='k', linestyle='--')

    fig_m_shell_P.savefig('plots/m_shell_P_of_r_plots/'+actual_plot_name_m_shell_P+'.png', dpi=resolution_dpi, bbox_inches='tight')
    plt.close(fig_m_shell_P)
      

# Das im folgenden auskommentierte macht Massenschalen- und Druck Plots getrennt, also nicht mit
# gemeinsamer x-Achse.


#        #--------------------#
#        # Massenschalen-Plot #
#        #--------------------#
#    fig_m_shell = plt.figure(actual_plot_name_mass_shell)
#    
#    plt.plot(TOV_data[0],TOV_data[2])
#    
#    plt.title(title_mass_shell_plot)
#    plt.xlabel(x_label_r,            fontsize=fontsize_axes_labels)
#    plt.ylabel(y_label_mass_shell, fontsize=fontsize_axes_labels)
#    
#        # Lege Achsenbereiche fest:
#    plt.xlim(0.0,R_max)
#    plt.ylim(0.0,M_max)
#    
#        # zeichne Legende:
#    ax = plt.gca()
#    ax.text(0.05, 0.95, textbox_content, transform=ax.transAxes, fontsize=14,
#    verticalalignment='top', bbox=textbox_properties)
#        # zeichne vertikale Linie an Stelle des aktuellen Stern-Radius:
#    ax.axvline(MR_data[0][i], color='k', linestyle='--')
#
#    #plt.show(fig_P)
#    fig_m_shell.savefig('plots/mass_shell_plots/'+actual_plot_name_mass_shell+'.png', dpi=resolution_dpi, bbox_inches='tight')
#    plt.close(fig_m_shell)
#    
#    
#    
#        #------------#
#        # Druck-Plot #
#        #------------#
#    fig_P = plt.figure(actual_plot_name_P)
#    
#    plt.plot(TOV_data[0],TOV_data[3])
#    
#    plt.title(title_pressure_plot)
#    plt.xlabel(x_label_r,          fontsize=fontsize_axes_labels)
#    plt.ylabel(y_label_pressure, fontsize=fontsize_axes_labels)
#    
#        # Lege Achsenbereiche fest:
#    plt.xlim(0.0,R_max) 
#    plt.ylim(0.0,P_max)
#    
#        # zeichne Legende:
#    ax = plt.gca()
#    ax.text(0.75, 0.95, textbox_content, transform=ax.transAxes, fontsize=14,
#    verticalalignment='top', bbox=textbox_properties)
#        # zeichne vertikale Linie an Stelle des aktuellen Stern-Radius:
#    ax.axvline(MR_data[0][i], color='k', linestyle='--')
#
#    #plt.show(fig_P)
#    #plt.show(fig_P)
#    fig_P.savefig('plots/pressure_plots/'+actual_plot_name_P+'.png', dpi=resolution_dpi, bbox_inches='tight')
#    plt.close(fig_P)
#    
    ####################
    # Querschnit-Plots #
    ####################
    
    
    def get_coordinate_columns(TOV_data):
        TOV_data_cartesian = [[],[],[],[],[]]
        phi = np.arange(0,np.pi/2.0,0.01)
        for i in range(0,len(TOV_data[0])):
            for j in range(0,len(phi)):
                TOV_data_cartesian[0].append(TOV_data[0][i]*np.cos(phi[j]))
                TOV_data_cartesian[1].append(TOV_data[0][i]*np.sin(phi[j]))
                TOV_data_cartesian[2].append(TOV_data[1][i])
                TOV_data_cartesian[3].append(TOV_data[2][i])
                TOV_data_cartesian[4].append(TOV_data[3][i])
        return TOV_data_cartesian
    
    
    #-----------------------------------------#
    # Sehr zeitintensiv, daher auskommentiert #
    #-----------------------------------------#
    #fig_scatter_rho = plt.figure("scatter_plot_rho")
    #
    #TOV_data_cartesian = get_coordinate_columns(TOV_data)
    #
    #rho_scatter = plt.scatter( TOV_data_cartesian[0], TOV_data_cartesian[1], marker='.', s=dotsize_scatter,\
    #                           linewidths=0, c=TOV_data_cartesian[2], cmap='jet', norm=norm_quarter_star ) # alternatively norm = LogNorm()
    #plt.xlabel("x")
    #plt.ylabel("y")
    #
    #    #ax = plt.gca()
    #    #
    #    #ax.set_xscale('log')
    #    #ax.set_yscale('log')
    #plt.colorbar()
    #
    #plt.show()
    #-----------------------------------------#
    
